C Copyright: H. K. D. H. Bhadeshia, University of Cambridge
C Added to mucg64 to cope with direct acicular ferrite calculation
C Jan 1991 Program for the calculation of TTT curves for bainite
C and the microstructure of steels cooled at a constant cooling rate
C - mixed microstructures of martensite, bainitic ferrite & retained austenite.
C
C First calculates a series of TTT curves for bainite with a variety of
C austenite carbon concentrations.
C
C Reference: M. Takahashi and H. K. D. H. Bhadeshia
C  Materials Transactions of the Japan Institute of Metals
C  Vol. 32 (1991) 689-696.
C
C This subroutine is designed for welding problems involving
C  continuous cooling transformation, the cooling curves being
C  determined by the parameters C1, C2, CURR, VOLT, EFF, SPEED
C  and TINT which are inputs to this subroutine. C1 and C2 are
C  heat flow constants which depend on the welding process. CURR is
C  the welding current. VOLT is the welding voltage. EFF is the arc
C  transfer efficiency. SPEED is the welding speed in m/s. TINT
C  is the interpass or preheat temperature in degrees centigrade.
C
C COOL is the cooling rate in K/s
C JT sets the temperature interval (reduce to improve accuracy)
C TCAMB is the ambient temperature for retained austenite calculation
C IF JRICH = 1 residual austenite C-enriched due to prior reaction
C
C Version No. = 1.3/1.4 (JCALC and JT increased to 40 on inclusion in MAP)
C
C Modification Nov 1992: skips calculations if VOLFRA+VOLW>0.99 since
C there is no austenite left to transform in that circumstance.
C
C AVOLB is the volume fraction of bainitic ferrite
C VMART is the volume fraction of martensite (Khan & Bhadeshia)
C VMART2 is the volume fraction of martensite (Koistinen & Marburger)
C VAUSTR is the volume fraction of retained austenite (Khan & Bhadeshia)
C VAUST2 is the volume fraction of retained austenite (Koistinen & Marburger)
C VOLFRA is the volume fraction of allotriomorphic ferrite (an input)
C VOLW is the volume fraction of Widmanstatten ferrite (an input)
C
C Reference for Martensite: S. A. Khan and H. K. D. H. Bhadeshia,
C  Materials Science and Engineering A, A129 (1990) 257-272
C
      SUBROUTINE MAP_STEEL_BAINCA(CC,TINT,C1,C2,CURR,VOLT,EFF,
     &           SPEED,AVOLB,VMART,VAUSTR,VMART2,VAUST2,
     &           VOLFRA,VOLW,W1,JRICH)
C
      IMPLICIT NONE
      DOUBLE PRECISION AVOLB,CC(8),C1,C2,CURR,EFF,SPEED,TINT
      DOUBLE PRECISION VAUSTR,VAUST2,VMART,VMART2,VOLFRA,VOLW
      DOUBLE PRECISION W1,VOLT
      INTEGER JRICH
      DOUBLE PRECISION C(8),AC(8),KTEMP(200),SHEART(200)
      DOUBLE PRECISION CTEMP(100,200),SHEATT(100,200),DT(100)
      DOUBLE PRECISION BST(100),VOLB22(100),MST(100)
      DOUBLE PRECISION ADUM,BS,BXTO,CONST,COOL,CXTO,MS,R,SLOPE
      DOUBLE PRECISION T10,T20,TCAMB,VAUST,VOLB,VOLB2
      DOUBLE PRECISION W,XBAR,XTOCN,XTOCN2,XTOSL,XTOSL2
      DOUBLE PRECISION MAP_STEEL_CULRAT,MAP_STEEL_XALPH
      INTEGER I,IA,IB,IC,J,JCALC,JNO,JNUM,JT,JTHIGH,JTLOW,JY(100)
C
C Physical constants used in calculations.
C Reference : "Fundamental physical constants and energy conversion factors",
C National Physical Laboratory.
C
C  R is the molar gas constant (in joules per mole per kelvin, J/(mol K))
C
      R=8.31451D+00
C
      IF (VOLFRA+VOLW .GE. 0.99D+00) THEN
         AVOLB=0.0D+00
         VMART=0.0D+00
         VMART2=0.0D+00
         VAUSTR=0.0D+00
         VAUST2=0.0D+00
         GOTO 180
      ENDIF
C
C Modification May 1991: JTLOW raised to 473 from 373. Otherwise problems in
C  GMAAX associated with X values much less than 1D-09. Ensure JTLOW is below
C  Ms of residual austenite.
C
C JCALC represents the number of 'C' curves computed for the
C  bainite reaction, as a function of the extent of reaction
C  It has a maximum value of 100, set by matrix dimensions.
C  Each 'C' curve represents transformation from austenite whose
C  carbon concentration is enriched by 0.05 wt.%
C
      JCALC=40
      JT=40
C
C Calculations only cover the temperature range JTHIGH to JTLOW
C  It is assumed that bainite will never form above 873 K and never
C  form below 473 K
C
      JTHIGH=873
      JTLOW=473.0D+00
C
C TCAMB is the assumed ambient temperature in centigrade
C
      TCAMB=20.0D+00
C
      JNUM=(JTHIGH-JTLOW)/JT
C
      DO 100 IC=1,8
         C(IC)=CC(IC)
  100 CONTINUE
C
C carbon in ferrite in wt. % assumed 20*xalpha(873K)
C Following ensures austenite enriched in C due to prior VOLFRA & VOLW
C
      IF(JRICH .EQ. 1) C(1)=(C(1) - 20.0D+00*(VOLFRA+VOLW)
     &         *MAP_STEEL_XALPH(873.0D+00))/(1.0D+00-VOLFRA-VOLW)
C
C      -----------------------------------------------------
C           Begin calculating the C-curves for bainite
C
      DO 120 J=1,JCALC
         IF (J .GT. 1) THEN
            C(1)=C(1)+0.05D+00
         ENDIF
C
C Calculate the 'C' curve for bainite, using carbon
C concentration given by the current value of C(1).
C This also gives values of T10, T20, W which
C are required for later calculations.
C
         CALL MAP_STEEL_BAINTT(KTEMP,SHEART,C,AC,JT,JTHIGH,JTLOW,
     &                      T10,T20,W,W1,R,JNO)
C
C
C Calculate the Bs (bainite-start) and Ms (Martensite-start)
C  temperatures for the austenite, in degrees centigrade.
C Note that the slopes XTOSL and XTOSL2, and the constants
C XTOCN and XTOCN2 are for the T-zero and T-zero' curves
C respectively of the Temperature versus Carbon plot on
C the paraequilibrium phase diagram.
C
         IF(J .EQ. 1) XBAR=AC(1)
         CALL MAP_STEEL_BSMS(T10,T20,W,W1,JTLOW,JTHIGH,JT,BS,MS,
     &          AC(1),XTOSL,XTOCN,XTOSL2,XTOCN2,R)
         MST(J)=MS
         BST(J)=BS
C
C Low Ms or Ms below Interpass temperature not tolerated
         IF(MS.LT.30.0D+00 .OR. MS.LT.TINT+20.0D0) GOTO 130
C
C Calculate the limiting fractions of bainitic ferrite,
C   given the T-zero and T-zero' curves. Apply the lever
C   rule to these curves, the carbon concentration of the
C   bainitic ferrite given by the function MAP_STEEL_XALPH.
C BXTO and CXTO are the T-zero and T-zero' carbon
C   concentrations respectively.
C
         BXTO=BS*XTOSL+XTOCN
         VOLB=(BXTO-XBAR)/(BXTO-MAP_STEEL_XALPH(BS+273.0))
         CXTO=BS*XTOSL2+XTOCN2
         VOLB2=(CXTO-XBAR)/(CXTO-MAP_STEEL_XALPH(BS+273.0))
         VOLB22(J)=VOLB2
C
         DO 110 I=1,JNUM
            IF (KTEMP(I) .GT. BS +273.0 .OR.
     &          KTEMP(I) .LT. MS+273.0+20.00) THEN
               CTEMP(J,I)=0.0D+00
               SHEATT(J,I)=0.0D+00
            ELSE
               CTEMP(J,I)=KTEMP(I)-273.0D+00
               SHEATT(J,I)=SHEART(I)
            ENDIF
  110    CONTINUE
  120 CONTINUE
C
C       Calculation of C-Curves for bainite complete
C       --------------------------------------------
C
  130 DO 150 IA=1,JCALC
         DT(IA)=0.0D+00
C
C Initialize JY to zero to represent Scheil parameter < 1
C
         JY(IA)=0
C
C Schiel's rule
C Done with lower temperatures first in order to avoid crashes
C
         DO 140 IB=JNUM,1,-1
            IF (CTEMP(IA,IB) .EQ. 0.0D+00) THEN
               GOTO 140
            ELSE
               COOL=MAP_STEEL_CULRAT(CTEMP(IA,IB),TINT,C1,C2,CURR,
     &                               VOLT,EFF,SPEED)
C
C add to Scheil parameter for continuous cooling
C transformation. Set JY=1 if Scheil parameter > 1
C
                DT(IA)=(JT/COOL)/SHEATT(IA,IB)+DT(IA)
                IF (DT(IA) .GE. 1.0D+00) JY(IA)=1
            ENDIF
C
  140    CONTINUE
C
         IF(JY(IA) .EQ. 0) GOTO 160
  150 CONTINUE
C
  160 IF (IA .EQ. 1 .AND. JY(IA) .EQ. 0) THEN
         AVOLB=0.0D+00
         GOTO 170
      ENDIF
C
C Find the actual volume fraction of bainite corresponding
C  to the case where Scheil parameter equal unity exactly
C
      IF (VOLB22(IA) .GT. 0.0D+00 .AND. DT(IA) .GT. 0.0D+00) THEN
         SLOPE=(VOLB22(IA)-VOLB22(IA-1))/(DT(IA)-DT(IA-1))
         CONST=VOLB22(IA)-SLOPE*DT(IA)
      ELSE
         SLOPE=(VOLB22(IA-1)-VOLB22(IA-2))/(DT(IA-1)-DT(IA-2))
         CONST=VOLB22(IA-1)-SLOPE*DT(IA-1)
      ENDIF
      AVOLB=SLOPE+CONST
C
C Modification October 1992
C
      IF (AVOLB .GT. 1.0D+00) AVOLB=1.0D+00
C
  170 VAUST=1.0D+00-AVOLB
C
C Calculate martensite-start temperatures -
C  two methods, Khan & Bhadeshia and Koistenen & Marburger
C
      CALL MAP_STEEL_MART(MST(IA),TCAMB,VAUST,VMART,VAUSTR)
      CALL MAP_STEEL_MART2(MST(IA),TCAMB,VAUST,VMART2,VAUST2)
C
C     WRITE(*,1) AVOLB,VMART,VAUSTR,VMART2,VAUST2,MST(IA)
C
C Normalise to allow for prior transformation
C
      ADUM=1.0D+00-VOLFRA-VOLW
      AVOLB=AVOLB*ADUM
      VMART=VMART*ADUM
      VAUSTR=VAUSTR*ADUM
      VAUST2=VAUST2*ADUM
      VMART2=VMART2*ADUM
      IF (MST(IA) .LT. JTLOW-273) THEN
         WRITE(*,2)
      ENDIF
  180 RETURN
    1 FORMAT(/' The following do not allow for allotriomorphic'/
     &        ' and Widmanstatten ferrite which are included later'//
     &       ' Volume fraction bainite            =',5X,F7.2/
     &       ' Volume fraction martensite         =',5X,F7.2,
     &' (Khan & Bhadeshia)'/
     &       ' Volume fraction retained austenite =',5X,F7.2,
     &' (Khan & Bhadeshia)'/
     &       ' Volume fraction martensite         =',5X,F7.2,
     &' (Koistinen & Marburger)'/
     &       ' Volume fraction retained austenite =',5X,F7.2,
     &' (Koistinen & Marburger)'//
     &       ' Ms of residual austenite,     C    =',5X,F7.2/)
    2 FORMAT(
     & //20X,' **********      WARNING     *************'/
     & / 20X,' Ms of retained austenite less than  JTLOW '/
     & / 20X,' Volume fraction of bainite underestimated'//)
      END
